﻿// PIDController.cs
// 
//  Modified MIT License (MIT)
//  
//  Copyright (c) 2015 Completely Fair Games Ltd.
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// The following content pieces are considered PROPRIETARY and may not be used
// in any derivative works, commercial or non commercial, without explicit 
// written permission from Completely Fair Games:
// 
// * Images (sprites, textures, etc.)
// * 3D Models
// * Sound Effects
// * Music
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace DwarfCorp
{
    /// <summary>
    /// A simple way of applying forces to things to move them around smoothly.
    /// Very robust and easy to create. Uses a proportional (P), Integral (I) and Derivative (D)
    /// term to control the force.
    /// </summary>
    public class PIDController
    {
        public float Kp { get; set; }
        public float Kd { get; set; }
        public float Ki { get; set; }

        public Vector3 LastError { get; set; }
        public Vector3 SumError { get; set; }

        public PIDController(float kp, float kd, float ki)
        {
            Kp = kp;
            Kd = kd;
            Ki = ki;

            LastError = Vector3.Zero;
            SumError = Vector3.Zero;
        }

        public void Reset()
        {
            LastError = Vector3.Zero;
            SumError = Vector3.Zero;
        }

        public Vector3 GetOutput(float dt, Vector3 target, Vector3 position)
        {
            Vector3 toReturn = Vector3.Zero;
            Vector3 error = (target - position);
            toReturn += Kp * error;
            toReturn += Kd * (error - LastError);
            toReturn += Ki * (SumError);


            LastError = error;
            SumError += error;

            return toReturn;
        }
    }

}